<?php

namespace Zen\Paymentpage\Model;

include_once __DIR__ . "/../Libs/payment-core/src/Util.php";

use Magento\Framework\Api\AttributeValueFactory;
use Magento\Framework\Api\ExtensionAttributesFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Payment\Helper\Data;
use Magento\Payment\Model\Method\Logger;
use Magento\Sales\Model\Order;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Zen\Payment\Api;
use Zen\Payment\Util;
use Zen\Paymentpage\Model\Payment\AbstractPaymentGateway;

/**
 * Class PaymentPage
 *
 * @package Zen\Paymentpage\Model
 */
class Paymentpage extends AbstractPaymentGateway
{
	const CODE = 'zen_paymentpage';

	/**
	 * @var UrlInterface
	 */
	protected $urlBuilder;

	/**
	 * @var string
	 */
	protected $code = self::CODE;

	/**
	 * @var Order
	 */
	private $order;
	/**
	 * @var StoreManagerInterface
	 */
	private $storeManager;

	/**
	 * PaymentPage constructor.
	 *
	 * @param Context $context
	 * @param Registry $registry
	 * @param ExtensionAttributesFactory $extensionFactory
	 * @param AttributeValueFactory $customAttributeFactory
	 * @param Data $paymentData
	 * @param ScopeConfigInterface $scopeConfig
	 * @param Logger $logger
	 * @param UrlInterface $urlBuilder
	 * @param Order $order
	 * @param StoreManagerInterface $storeManager
	 * @param AbstractResource|null $resource
	 * @param AbstractDb|null $resourceCollection
	 * @param array $data
	 */
	public function __construct(
		Context                    $context,
		Registry                   $registry,
		ExtensionAttributesFactory $extensionFactory,
		AttributeValueFactory      $customAttributeFactory,
		Data                       $paymentData,
		ScopeConfigInterface       $scopeConfig,
		Logger                     $logger,
		UrlInterface               $urlBuilder,
		Order                      $order,
		StoreManagerInterface      $storeManager,
		AbstractResource           $resource = null,
		AbstractDb                 $resourceCollection = null,
		array                      $data = []
	)
	{
		parent::__construct(
			$context,
			$registry,
			$extensionFactory,
			$customAttributeFactory,
			$paymentData,
			$scopeConfig,
			$logger,
			$resource,
			$resourceCollection,
			$data
		);

		$this->urlBuilder = $urlBuilder;
		$this->order = $order;
		$this->storeManager = $storeManager;

		$this->setStore($this->storeManager->getStore()->getId());
	}

	/**
	 * @return string
	 */
	public function getCheckoutRedirectUrl()
	{

		return $this->urlBuilder->getUrl('zen_paymentpage/payment/checkout');
	}

	/**
	 * @return bool
	 */
	public function canRefund()
	{
		return true;
	}

	/**
	 * @return string
	 */
	public function getPluginVersion()
	{
		return $this->getConfigData('version');
	}

	/**
	 * @return string
	 */
	public function getPluginName()
	{
		return $this->getConfigData('name');
	}

	/**
	 * @param \Magento\Payment\Model\InfoInterface $payment
	 * @param float $amount
	 *
	 * @return void|Paymentpage
	 */
	public function refund(\Magento\Payment\Model\InfoInterface $payment, $amount)
	{
		$api = new Api(
			$this->getApiKey(),
			$this->getEnvironment()
		);

		$order = $payment->getOrder();

		$refund = $api->createRefund(
			$amount,
			$payment->getParentTransactionId(),
			$order->getOrderCurrency()->getCode(),
			$order->getIncrementId()
		);

		if ($refund['success']) {

			$msg = __('Refund for amount:')
				. ' '
				. $amount
				. ' '
				. $order->getOrderCurrency()->getCode()
				. ' '
				. __('with Zen UUID:')
				. ' '
				. $refund['body']['id']
				. ' '
				. __('has been requested and waiting for completion.');

			throw new \Magento\Framework\Exception\LocalizedException(__($msg));
		}

		throw new \Magento\Framework\Exception\LocalizedException(__('The refund action is not available at this moment. Try again later.'));
	}

	/**
	 * @return string
	 */
	public function getApiKey()
	{
		return $this->getConfigData('api_secret');
	}

	/**
	 * @return string
	 */
	public function getEnvironment()
	{
		return Util::ENVIRONMENT_PRODUCTION;
	}

	/**
	 * @param string $currencyCode
	 *
	 * @return bool
	 */
	public function canUseForCurrency($currencyCode)
	{
		$currency = $this->getConfigData('currency');

		if (!is_string($currency)) {
			return false;
		}

		return Util::canUseForCurrency(
				strtolower($currencyCode),
				explode(
					',',
					strtolower(
						str_replace(' ', '', $currency)
					)
				)
			)
			&& $this->getTerminalId()
			&& $this->getPaywallSecret();
	}

	/**
	 * @return string
	 */
	public function getTerminalId()
	{
		return $this->getConfigData('terminal_id');
	}

	/**
	 * @return string
	 */
	public function getPaywallSecret()
	{
		return $this->getConfigData('paywall_secret');
	}

	/**
	 * @return mixed
	 */
	public function getLocale()
	{
		return $this->scopeConfig->getValue('general/locale/code', ScopeInterface::SCOPE_STORE, $this->getStore());
	}

}
